use crate::error::Result;
use pyo3::PyAny;

pub trait PyAnyExtras {
    fn qualified_name(&self) -> Result<String>;
    fn qualified_class_name(&self) -> Result<String>;
    fn get_proto_meta(&self) -> Result<&PyAny>;
    fn get_class(&self, field_name: &str) -> Result<&PyAny>;
    fn create_instance(&self, field_name: &str) -> Result<&PyAny>;
    fn is_list_field(&self, field_name: &str) -> Result<bool>;
    fn oneof_group(&self, field_name: &str) -> Result<Option<String>>;
}

impl PyAnyExtras for PyAny {
    fn qualified_name(&self) -> Result<String> {
        let module = self.getattr("__module__")?;
        let name = self.getattr("__name__")?;
        Ok(format!("{module}.{name}"))
    }

    fn qualified_class_name(&self) -> Result<String> {
        self.getattr("__class__")?.qualified_name()
    }

    fn get_proto_meta(&self) -> Result<&PyAny> {
        Ok(self.getattr("_betterproto")?)
    }

    fn get_class(&self, field_name: &str) -> Result<&PyAny> {
        let cls = self.getattr("cls_by_field")?.get_item(field_name)?;
        Ok(cls)
    }

    fn create_instance(&self, field_name: &str) -> Result<&PyAny> {
        Ok(self.get_class(field_name)?.call0()?)
    }

    fn is_list_field(&self, field_name: &str) -> Result<bool> {
        let cls = self.getattr("default_gen")?.get_item(field_name)?;
        let module = cls.getattr("__module__")?;
        let name = cls.getattr("__name__")?;
        Ok(module.to_string() == "builtins" && name.to_string() == "list")
    }

    fn oneof_group(&self, field_name: &str) -> Result<Option<String>> {
        let opt = self
            .getattr("oneof_group_by_field")?
            .call_method1("get", (field_name,))?
            .extract()?;
        Ok(opt)
    }
}
